"
I'm Twitt 

I store twitts from twitter (http://twitter.com).

"
Class {
	#name : #Tweet,
	#superclass : #Object,
	#instVars : [
		'date',
		'message',
		'profile',
		'url'
	],
	#category : #'Dataviz-Twitter'
}

{ #category : #accessing }
Tweet >> date [
	^ date
]

{ #category : #accessing }
Tweet >> date: anObject [
	date := anObject
]

{ #category : #'data scrapping' }
Tweet >> impactFor: aTweetUrl [
	"Computes the maximum amount of followers that a tweet can reach, including the retweets.
	 It doesn't take into account that retweeters can share followers and in future versions
	 this should be computed, even if we don't have a full list of followers. 
	 Also the computing time could be sustancially improved by extracting only followers data 
	 from the profiles instead of scrapping all and by having a better cache which includes 	 	 
	 prescrapped data.
	 'Real time' is not the main issue here, but 'slow data'. 
	 For example we could include snapshots of the retweeters profiles and to aks for a 
	 commented retweet to reach new followers"

	| retweetsHtml retweetsTree retweeters impactData |

	retweetsHtml := ZnClient new get: 				'http://mutabit.com/deltas/repos.fossil/datapolis/doc/tip/Data/Sources/Single/', 				((aTweetUrl splitOn: '//') at: 2), '/retweets.html'.

	retweetsTree := Soup fromString: retweetsHtml.

	retweeters := (retweetsTree findAllTagsByClass: 'username') 
							collect: [:each | each text copyReplaceAll: '@' with: ''].
							
	impactData := Dictionary new.
	
	impactData 
		at: 'retweeters' put: retweeters size;
		at: 'reach' put: (retweeters collect: [:each | (TwitterProfile new scrapDataFromProfile: each) followers]) sum.

	^ impactData.
]

{ #category : #initialization }
Tweet >> initialize [
	"Creates a new TwitterProfile object"
	super initialize.
]

{ #category : #'data visualization' }
Tweet >> labeledCircleSized: diameter upperLabel: label1 bottomLabel: label2 inView: v [

| circle ex1 ex2 line label1Temp label2Temp radious  origin background els |
 
background := (RTBox new color: Color transparent) element.
v add: background.

origin := (0@0).
radious := diameter / 2.
 
circle := (RTEllipse new size: diameter"; borderColor: Color black; color: Color transparent") element.
ex1 := RTBox element.
ex2 := RTBox element.
circle translateTo: origin.
ex1 translateTo: ((radious negated @ 0) + circle center).
ex2 translateTo: (radious @ 0) + circle center.
line := (RTLine new; color: Color black) edgeFrom: ex1 to: ex2.
v add: circle; add: line.
 
label1Temp := (RTLabel text: label1) element.
label2Temp  := (RTLabel text: label2) element.
v add: label1Temp ; add: label2Temp .
label1Temp  translateTo: (0 @ -12) + circle center.
label2Temp  translateTo: (0 @ 12) + circle center.

els := RTGroup with: label1Temp  with: label2Temp with: circle with: ex1 with: ex2.
RTNest new on: background nest: els.

^ background
]

{ #category : #accessing }
Tweet >> message [
	^ message
]

{ #category : #accessing }
Tweet >> message: anObject [
	message := anObject
]

{ #category : #accessing }
Tweet >> profile [
	^ profile
]

{ #category : #accessing }
Tweet >> profile: anObject [
	profile := anObject
]

{ #category : #'data visualization' }
Tweet >> silenceMap [
	"Creates a visualization of how long a tweet has not been answered (any kind of answer: not favs, not RT, no nothing), as
	'circles of silence' "

	| v circle ex1 ex2 line retweets reach diameter radious  origin |

diameter := 60.
origin := (100@100).
radious := diameter / 2.
v := RTView new.

circle := (RTEllipse new size: diameter; borderColor: Color black; color: Color transparent) element.
ex1 := RTBox element.
ex2 := RTBox element.
circle translateTo: origin.
ex1 translateTo: ((radious negated @ 0) + circle center).
ex2 translateTo: (radious @ 0) + circle center.
line := (RTLine new; color: Color black) edgeFrom: ex1 to: ex2.
v add: circle; add: line.

retweets := (RTLabel text: 5) element.
reach := (RTLabel text: '3.1k') element.
v add: retweets; add: reach. 
retweets translateTo: (0 @ -12) + circle center.
reach translateTo: (0 @ 12) + circle center.
^ v.
]

{ #category : #accessing }
Tweet >> url [
	^ url
]

{ #category : #accessing }
Tweet >> url: anObject [
	url := anObject
]
